@@ -8,7 +8,7 @@ def generate(args):
 	start = time.time()
 	generator = args.generator
 	m = importlib.import_module('.'+generator, package = 'vinca.generators')
-	new_cards = m.generate()
+	new_cards = m.generate(args)
 	stop = time.time()
 	elapsed = int(stop - start)
 	
@@ -16,5 +16,6 @@ def generate(args):
 		# card.tags = card.tags + args.tags
 		card.make_string()
 		card.add_history(TODAY, elapsed, 'create')
+		card.load_metadata()  # TODO why is this necessary?
 
 	return new_cards
